@@ -16,12 +16,12 @@
 
 from __future__ import annotations
 
+import logging
 from abc import ABCMeta, abstractmethod
 from collections.abc import AsyncIterable, Awaitable, Iterable
 from dataclasses import dataclass
 from datetime import timedelta
 from enum import Enum
-import logging
 from typing import Any, Callable, TypedDict, final
 
 from aiohomekit.characteristic_cache import CharacteristicCacheType
@@ -73,9 +73,7 @@ class AbstractPairing(metaclass=ABCMeta):
     # and BLE advertisements), and also as AccessoryPairingID i pairing data.
     id: str
 
-    def __init__(
-        self, controller: AbstractController, pairing_data: AbstractPairingData
-    ) -> None:
+    def __init__(self, controller: AbstractController, pairing_data: AbstractPairingData) -> None:
         self.controller = controller
         self.listeners: set[Callable[[dict], None]] = set()
         self.subscriptions: set[tuple[int, int]] = set()
@@ -149,7 +147,7 @@ class AbstractPairing(metaclass=ABCMeta):
         return self._accessories_state.state_num
 
     @abstractmethod
-    async def _process_disconnected_events(self):
+    def _process_disconnected_events(self):
         """Process any disconnected events that are available."""
 
     def _callback_listeners(self, event):
@@ -160,9 +158,7 @@ class AbstractPairing(metaclass=ABCMeta):
             except Exception:
                 logger.exception("Unhandled error when processing event")
 
-    def _async_description_update(
-        self, description: AbstractDescription | None
-    ) -> None:
+    def _async_description_update(self, description: AbstractDescription | None) -> None:
         if self._shutdown:
             return
 
@@ -185,10 +181,7 @@ class AbstractPairing(metaclass=ABCMeta):
                 )
                 repopulate_accessories = True
 
-            elif (
-                not self.description
-                or description.state_num != self.description.state_num
-            ):
+            elif not self.description or description.state_num != self.description.state_num:
                 # Only process disconnected events if the config number has
                 # not also changed since we will do a full repopulation
                 # of the accessories anyway when the config number changes.
@@ -203,7 +196,7 @@ class AbstractPairing(metaclass=ABCMeta):
                     "%s: Disconnected event notification received; Triggering catch-up poll",
                     self.name,
                 )
-                async_create_task(self._process_disconnected_events())
+                self._process_disconnected_events()
 
         self.description = description
 
@@ -212,9 +205,7 @@ class AbstractPairing(metaclass=ABCMeta):
 
     def _load_accessories_from_cache(self) -> None:
         if (cache := self.controller._char_cache.get_map(self.id)) is None:
-            logger.debug(
-                "%s: No accessories cache available for this device", self.name
-            )
+            logger.debug("%s: No accessories cache available for this device", self.name)
             return
         config_num = cache.get("config_num", 0)
         state_num = cache.get("state_num")
@@ -243,9 +234,7 @@ class AbstractPairing(metaclass=ABCMeta):
     ) -> None:
         """Restore accessories from cache."""
         accessories = Accessories.from_list(accessories)
-        self._accessories_state = AccessoriesState(
-            accessories, config_num, broadcast_key, state_num
-        )
+        self._accessories_state = AccessoriesState(accessories, config_num, broadcast_key, state_num)
         self._update_accessories_state_cache()
 
     def _update_accessories_state_cache(self):
@@ -266,9 +255,7 @@ class AbstractPairing(metaclass=ABCMeta):
         else:
             parsed = self.accessories
 
-        accessory_info = parsed.aid(1).services.first(
-            service_type=ServicesTypes.ACCESSORY_INFORMATION
-        )
+        accessory_info = parsed.aid(1).services.first(service_type=ServicesTypes.ACCESSORY_INFORMATION)
         return accessory_info.value(CharacteristicsTypes.NAME, "")
 
     @abstractmethod
@@ -303,12 +290,12 @@ class AbstractPairing(metaclass=ABCMeta):
     @abstractmethod
     async def get_characteristics(
         self,
-        characteristics,
-        include_meta=False,
-        include_perms=False,
-        include_type=False,
-        include_events=False,
-    ):
+        characteristics: Iterable[tuple[int, int]],
+        include_meta: bool = False,
+        include_perms: bool = False,
+        include_type: bool = False,
+        include_events: bool = False,
+    ) -> dict[tuple[int, int], dict[str, Any]]:
         """Get characteristics."""
 
     @abstractmethod
@@ -333,6 +320,7 @@ class AbstractPairing(metaclass=ABCMeta):
     async def _shutdown_if_primary_pairing_removed(self, pairingId: str) -> None:
         """Shutdown the connection if the primary pairing was removed."""
         if pairingId == self._pairing_data.get("iOSPairingId"):
+            logger.debug("%s: Primary pairing removed; shutting down", self.name)
             await self.shutdown()
 
     async def shutdown(self) -> None:
@@ -342,6 +330,7 @@ class AbstractPairing(metaclass=ABCMeta):
         the pairing is removed or the controller is shutdown.
         """
         self._shutdown = True
+        logger.debug("%s: Shutting down", self.name)
         await self.close()
 
     def _callback_availability_changed(self, available: bool) -> None:
@@ -355,9 +344,7 @@ class AbstractPairing(metaclass=ABCMeta):
             callback(self.config_num)
         self._update_accessories_state_cache()
 
-    async def subscribe(
-        self, characteristics: Iterable[tuple[int, int]]
-    ) -> set[tuple[int, int]]:
+    async def subscribe(self, characteristics: Iterable[tuple[int, int]]) -> set[tuple[int, int]]:
         new_characteristics = set(characteristics) - self.subscriptions
         self.subscriptions.update(characteristics)
         return new_characteristics
@@ -365,9 +352,7 @@ class AbstractPairing(metaclass=ABCMeta):
     async def unsubscribe(self, characteristics: Iterable[tuple[int, int]]) -> None:
         self.subscriptions.difference_update(characteristics)
 
-    def dispatcher_availability_changed(
-        self, callback: Callable[[bool], None]
-    ) -> Callable[[], None]:
+    def dispatcher_availability_changed(self, callback: Callable[[bool], None]) -> Callable[[], None]:
         """Notify subscribers when availablity changes.
 
         Currently this only notifies when a device is seen as available and
@@ -380,9 +365,7 @@ class AbstractPairing(metaclass=ABCMeta):
 
         return stop_listening
 
-    def dispatcher_connect_config_changed(
-        self, callback: Callable[[int], None]
-    ) -> Callable[[], None]:
+    def dispatcher_connect_config_changed(self, callback: Callable[[int], None]) -> Callable[[], None]:
         """Notify subscribers of a new accessories state."""
         self.config_changed_listeners.add(callback)
 
@@ -391,9 +374,7 @@ class AbstractPairing(metaclass=ABCMeta):
 
         return stop_listening
 
-    def dispatcher_connect(
-        self, callback: Callable[[dict], None]
-    ) -> Callable[[], None]:
+    def dispatcher_connect(self, callback: Callable[[dict], None]) -> Callable[[], None]:
         """
         Register an event handler to be called when a characteristic (or multiple characteristics) change.
 
@@ -464,6 +445,10 @@ class AbstractController(metaclass=ABCMeta):
     async def async_find(self, device_id: str, timeout=10) -> AbstractDiscovery:
         """Find a device by id."""
 
+    @abstractmethod
+    async def async_reachable(self, device_id: str, timeout=10) -> bool:
+        """Check if a device is reachable on the network."""
+
     @abstractmethod
     async def async_discover(self, timeout=10) -> AsyncIterable[AbstractDiscovery]:
         """Discover all devices."""
