@@ -1,8 +1,8 @@
 from __future__ import annotations
 
 import asyncio
-from collections.abc import AsyncIterable
 import logging
+from collections.abc import AsyncIterable
 
 from bleak import BleakScanner
 from bleak.backends.device import BLEDevice
@@ -48,14 +48,12 @@ class BleController(AbstractController):
         self._scanner = bleak_scanner_instance
         self._ble_futures: dict[str, list[asyncio.Future[BLEDevice]]] = {}
 
-    def _device_detected(
-        self, device: BLEDevice, advertisement_data: AdvertisementData
-    ) -> None:
+    def _device_detected(self, device: BLEDevice, advertisement_data: AdvertisementData) -> None:
         manufacturer_data = advertisement_data.manufacturer_data
         if not (mfr_data := manufacturer_data.get(APPLE_MANUFACTURER_ID)):
             return
 
-        elif mfr_data[0] == HOMEKIT_ENCRYPTED_NOTIFICATION_TYPE:
+        if mfr_data[0] == HOMEKIT_ENCRYPTED_NOTIFICATION_TYPE:
             try:
                 data = HomeKitEncryptedNotification.from_manufacturer_data(
                     device.name, device.address, manufacturer_data
@@ -72,19 +70,14 @@ class BleController(AbstractController):
             return
 
         try:
-            data = HomeKitAdvertisement.from_manufacturer_data(
-                device.name, device.address, manufacturer_data
-            )
+            data = HomeKitAdvertisement.from_manufacturer_data(device.name, device.address, manufacturer_data)
         except ValueError:
             return
 
         if old_discovery := self.discoveries.get(data.id):
             if (old_name := old_discovery.description.name) and (
                 not (name := data.name)
-                or (
-                    old_name != old_discovery.device.address
-                    and len(old_name) > len(name)
-                )
+                or (old_name != old_discovery.device.address and len(old_name) > len(name))
             ):
                 #
                 # If we have a pairing and the name is longer than the one we
@@ -124,9 +117,7 @@ class BleController(AbstractController):
             try:
                 self._scanner = BleakScanner()
             except (FileNotFoundError, BleakDBusError, BleakError) as e:
-                logger.debug(
-                    "Failed to init scanner, HAP-BLE not available: %s", str(e)
-                )
+                logger.debug("Failed to init scanner, HAP-BLE not available: %s", str(e))
                 self._scanner = None
                 return
 
@@ -143,6 +134,13 @@ class BleController(AbstractController):
             self._scanner.register_detection_callback(None)
             self._scanner = None
 
+    async def async_reachable(self, device_id: str, timeout: float = 10) -> bool:
+        """Check if a device is reachable on the network."""
+        return bool(
+            (discovery := self.discoveries.get(device_id))
+            and discovery.device.address in self._scanner.discovered_devices_and_advertisement_data
+        )
+
     async def async_find(self, device_id: str, timeout: float = 10) -> BleDiscovery:
         if discovery := self.discoveries.get(device_id):
             logger.debug("Discovery for %s already found", device_id)
@@ -163,12 +161,10 @@ class BleController(AbstractController):
                 timeout,
                 device_id,
             )
-            raise AccessoryNotFoundError(
-                f"Accessory with device id {device_id} not found"
-            )
+            raise AccessoryNotFoundError(f"Accessory with device id {device_id} not found")
         finally:
             if device_id not in self._ble_futures:
-                return
+                return None
             if future in self._ble_futures[device_id]:
                 self._ble_futures[device_id].remove(future)
             if not self._ble_futures[device_id]:
@@ -178,9 +174,7 @@ class BleController(AbstractController):
         for device in self.discoveries.values():
             yield device
 
-    def load_pairing(
-        self, alias: str, pairing_data: AbstractPairingData
-    ) -> BlePairing | None:
+    def load_pairing(self, alias: str, pairing_data: AbstractPairingData) -> BlePairing | None:
         if pairing_data["Connection"] != "BLE":
             return None
 
@@ -195,9 +189,7 @@ class BleController(AbstractController):
             device = discovery.device
             description = discovery.description
 
-        pairing = self.pairings[id_] = BlePairing(
-            self, pairing_data, device=device, description=description
-        )
+        pairing = self.pairings[id_] = BlePairing(self, pairing_data, device=device, description=description)
         self.aliases[alias] = pairing
 
         return pairing
