@@ -1,8 +1,8 @@
 from __future__ import annotations
 
-from dataclasses import dataclass
 import logging
 import struct
+from dataclasses import dataclass
 
 from bleak.backends.device import BLEDevice
 from bleak.backends.scanner import AdvertisementData
@@ -23,7 +23,6 @@ logger = logging.getLogger(__name__)
 
 @dataclass
 class HomeKitAdvertisement(AbstractDescription):
-
     setup_hash: bytes
     address: str
     state_num: int
@@ -38,12 +37,16 @@ class HomeKitAdvertisement(AbstractDescription):
         if data[0] != HOMEKIT_ADVERTISEMENT_TYPE:
             raise ValueError("Not a HomeKit device")
 
+        # Minimum size is 15 bytes for the basic fields
+        # Some devices (like Eve gen1) may not include the setup_hash
+        if len(data) < 15:
+            raise ValueError("HomeKit advertisement data too short")
+
         sf = data[2]
-        device_id = ":".join(
-            data[3:9].hex()[0 + i : 2 + i] for i in range(0, 12, 2)
-        ).lower()
+        device_id = ":".join(data[3:9].hex()[0 + i : 2 + i] for i in range(0, 12, 2)).lower()
         acid, gsn, cn, cv = UNPACK_HHBB(data[9:15])
-        sh = data[15:19]
+        # Setup hash is optional - only present in data >= 19 bytes
+        sh = data[15:19] if len(data) >= 19 else b""
 
         return cls(
             name=name,
@@ -57,9 +60,7 @@ class HomeKitAdvertisement(AbstractDescription):
         )
 
     @classmethod
-    def from_cache(
-        cls, address: str, id: str, config_num: int, state_num: int
-    ) -> HomeKitAdvertisement:
+    def from_cache(cls, address: str, id: str, config_num: int, state_num: int) -> HomeKitAdvertisement:
         """Create a HomeKitAdvertisement from a cache entry."""
         return cls(
             name=address,
@@ -84,7 +85,6 @@ class HomeKitAdvertisement(AbstractDescription):
 
 @dataclass
 class HomeKitEncryptedNotification:
-
     name: str
     address: str
     id: str
@@ -102,9 +102,7 @@ class HomeKitEncryptedNotification:
             raise ValueError("Not a HomeKit encrypted notification")
 
         advertising_identifier = data[2:8]
-        device_id = ":".join(
-            advertising_identifier.hex()[0 + i : 2 + i] for i in range(0, 12, 2)
-        ).lower()
+        device_id = ":".join(advertising_identifier.hex()[0 + i : 2 + i] for i in range(0, 12, 2)).lower()
         encrypted_payload = data[8:]
 
         return cls(
