@@ -16,13 +16,14 @@
 from __future__ import annotations
 
 import asyncio
+import logging
 from collections.abc import Iterable
 from datetime import timedelta
 from itertools import groupby
-import logging
 from operator import itemgetter
 from typing import Any
 
+from aiohomekit import hkjson
 from aiohomekit.controller.abstract import AbstractController, AbstractPairingData
 from aiohomekit.exceptions import (
     AccessoryDisconnectedError,
@@ -33,7 +34,6 @@ from aiohomekit.exceptions import (
     UnknownError,
     UnpairedError,
 )
-import aiohomekit.hkjson as hkjson
 from aiohomekit.http import HttpContentTypes
 from aiohomekit.model import Accessories, AccessoriesState, Transport
 from aiohomekit.model.characteristics import (
@@ -55,9 +55,38 @@ logger = logging.getLogger(__name__)
 EMPTY_EVENT = {}
 
 
-def format_characteristic_list(data):
-    tmp = {}
-    for c in data["characteristics"]:
+def format_characteristic_list(
+    data: dict[str, Any], requested_characteristics: set[tuple[int, int]] | None = None
+) -> dict[tuple[int, int], dict[str, Any]]:
+    tmp: dict[tuple[int, int], dict[str, Any]] = {}
+
+    # Handle global error status first - set defaults for all requested characteristics
+    if "status" in data and data["status"] != 0:
+        # Device returned a global error status
+        status_code = to_status_code(data["status"])
+        # For unknown codes, include the actual code in the description
+        if status_code == HapStatusCode.UNKNOWN:
+            description = f"Unknown error code: {data['status']}"
+        else:
+            description = status_code.description
+
+        logger.debug(
+            "Device returned error status %s (%s) for characteristics request",
+            data["status"],
+            description,
+        )
+        # If we know what was requested, mark them all as failed initially
+        if requested_characteristics:
+            for aid, iid in requested_characteristics:
+                tmp[(aid, iid)] = {"status": data["status"], "description": description}
+
+    # Process any characteristics that are present - these override the defaults
+    for c in data.get("characteristics", []):
+        # Skip malformed characteristics (e.g., boolean values) or missing aid/iid
+        if not isinstance(c, dict) or "aid" not in c or "iid" not in c:
+            logger.debug("Skipping malformed characteristic: %s", c)
+            continue
+
         key = (c["aid"], c["iid"])
         del c["aid"]
         del c["iid"]
@@ -65,8 +94,13 @@ def format_characteristic_list(data):
         if "status" in c and c["status"] == 0:
             del c["status"]
         if "status" in c and c["status"] != 0:
-            c["description"] = to_status_code(c["status"]).description
+            status_code = to_status_code(c["status"])
+            if status_code == HapStatusCode.UNKNOWN:
+                c["description"] = f"Unknown error code: {c['status']}"
+            else:
+                c["description"] = status_code.description
         tmp[key] = c
+
     return tmp
 
 
@@ -75,9 +109,7 @@ class IpPairing(ZeroconfPairing):
     This represents a paired HomeKit IP accessory.
     """
 
-    def __init__(
-        self, controller: AbstractController, pairing_data: AbstractPairingData
-    ) -> None:
+    def __init__(self, controller: AbstractController, pairing_data: AbstractPairingData) -> None:
         """
         Initialize a Pairing by using the data either loaded from file or obtained after calling
         Controller.perform_pairing().
@@ -112,9 +144,11 @@ class IpPairing(ZeroconfPairing):
     @property
     def name(self) -> str:
         """Return the name of the pairing with the address."""
+        connection = self.connection
+        host = connection.connected_host or connection.hosts
         if self.description:
-            return f"{self.description.name} [{self.connection.host}:{self.connection.port}] (id={self.id})"
-        return f"[{self.connection.host}:{self.connection.port}] (id={self.id})"
+            return f"{self.description.name} [{host}:{connection.port}] (id={self.id})"
+        return f"[{host}:{connection.port}] (id={self.id})"
 
     def event_received(self, event):
         self._callback_listeners(format_characteristic_list(event))
@@ -130,34 +164,33 @@ class IpPairing(ZeroconfPairing):
             await self.subscribe(self.subscriptions)
 
     async def _ensure_connected(self):
-        if self._shutdown:
-            return
+        """Ensure we are connected to the device."""
         connection = self.connection
+        if self._shutdown or connection.is_connected:
+            return
+
         try:
             async with asyncio_timeout(10):
                 await connection.ensure_connection()
         except asyncio.TimeoutError:
             last_connector_error = connection.last_connector_error
-            if not last_connector_error or isinstance(
-                last_connector_error, asyncio.TimeoutError
-            ):
+            if not last_connector_error or isinstance(last_connector_error, asyncio.TimeoutError):
                 raise AccessoryDisconnectedError(
-                    f"Timeout while waiting for connection to device {self.connection.host}:{self.connection.port}"
+                    f"Timeout while waiting for connection to device {connection.hosts}:{connection.port}"
                 )
             # The exception name is included since otherwise the error message
             # is not very helpful as it could be something like `step 3`
             raise AccessoryDisconnectedError(
-                f"Error while connecting to device {self.connection.host}:{self.connection.port}: "
+                f"Error while connecting to device {connection.hosts}:{connection.port}: "
                 f"{last_connector_error} ({type(last_connector_error).__name__})"
             )
 
-        if not self.connection.is_connected:
+        if not connection.is_connected:
             raise AccessoryDisconnectedError(
-                f"Ensure connection returned but still not connected: {self.connection.host}:{self.connection.port}"
+                f"Ensure connection returned but still not connected: {connection.hosts}:{connection.port}"
             )
 
-        else:
-            self._callback_availability_changed(True)
+        self._callback_availability_changed(True)
 
     async def close(self) -> None:
         """
@@ -186,9 +219,7 @@ class IpPairing(ZeroconfPairing):
                 for characteristic in service["characteristics"]:
                     characteristic["type"] = normalize_uuid(characteristic["type"])
 
-        self._accessories_state = AccessoriesState(
-            Accessories.from_list(accessories), self.config_num or 0
-        )
+        self._accessories_state = AccessoriesState(Accessories.from_list(accessories), self.config_num or 0)
         self._update_accessories_state_cache()
         return accessories
 
@@ -217,10 +248,7 @@ class IpPairing(ZeroconfPairing):
         if not (data[0][0] == TLV.kTLVType_State and data[0][1] == TLV.M2):
             raise UnknownError("unexpected data received: " + str(data))
 
-        if (
-            data[1][0] == TLV.kTLVType_Error
-            and data[1][1] == TLV.kTLVError_Authentication
-        ):
+        if data[1][0] == TLV.kTLVType_Error and data[1][1] == TLV.kTLVError_Authentication:
             raise UnpairedError("Must be paired")
 
         tmp = []
@@ -242,8 +270,8 @@ class IpPairing(ZeroconfPairing):
 
     async def get_characteristics(
         self,
-        characteristics,
-    ):
+        characteristics: Iterable[tuple[int, int]],
+    ) -> dict[tuple[int, int], dict[str, Any]]:
         """
         This method is used to get the current readouts of any characteristic of the accessory.
 
@@ -265,13 +293,16 @@ class IpPairing(ZeroconfPairing):
         if not self.accessories:
             await self.list_accessories_and_characteristics()
 
-        url = "/characteristics?id=" + ",".join(
-            str(x[0]) + "." + str(x[1]) for x in set(characteristics)
-        )
+        if isinstance(characteristics, set):
+            characteristics_set = characteristics
+        else:
+            characteristics_set = set(characteristics)
+
+        url = "/characteristics?id=" + ",".join(f"{aid}.{iid}" for aid, iid in characteristics_set)
 
         response = await self.connection.get_json(url)
 
-        return format_characteristic_list(response)
+        return format_characteristic_list(response, characteristics_set)
 
     async def put_characteristics(
         self, characteristics: Iterable[tuple[int, int, Any]]
@@ -302,15 +333,21 @@ class IpPairing(ZeroconfPairing):
             if CharacteristicPermissions.paired_read in char.perms:
                 listener_update[(aid, iid)] = {"value": value}
 
-        response = await self.connection.put_json(
-            "/characteristics", {"characteristics": char_payload}
-        )
+        response = await self.connection.put_json("/characteristics", {"characteristics": char_payload})
         response_status: dict[tuple[int, int], dict[str, Any]] = {}
         if response:
             # If there is a response it means something failed so
             # we need to remove the listener update for the failed
             # characteristics.
             for characteristic in response["characteristics"]:
+                # Skip malformed characteristics (e.g., boolean values)
+                if (
+                    not isinstance(characteristic, dict)
+                    or "aid" not in characteristic
+                    or "iid" not in characteristic
+                ):
+                    logger.debug("Skipping malformed characteristic in response: %s", characteristic)
+                    continue
                 aid, iid = characteristic["aid"], characteristic["iid"]
                 key = (aid, iid)
                 status = characteristic["status"]
@@ -337,14 +374,12 @@ class IpPairing(ZeroconfPairing):
             logger.info(
                 "This device does not support push, so only polling operations will be supported during this session"
             )
-            return
+            return None
 
         try:
             await self._ensure_connected()
         except AccessoryDisconnectedError:
-            logger.debug(
-                "Attempted to subscribe to characteristics but could not connect to accessory"
-            )
+            logger.debug("Attempted to subscribe to characteristics but could not connect to accessory")
             return {}
 
         try:
@@ -389,10 +424,11 @@ class IpPairing(ZeroconfPairing):
             if response:
                 # An empty body is a success response
                 for row in response.get("characteristics", []):
-                    status[(row["aid"], row["iid"])] = {
-                        "status": row["status"],
-                        "description": to_status_code(row["status"]).description,
-                    }
+                    if "aid" in row and "iid" in row:
+                        status[(row["aid"], row["iid"])] = {
+                            "status": row["status"],
+                            "description": to_status_code(row["status"]).description,
+                        }
 
         return status
 
@@ -413,12 +449,10 @@ class IpPairing(ZeroconfPairing):
         This method is called when the config num changes.
         """
         await self.list_accessories_and_characteristics()
-        self._accessories_state = AccessoriesState(
-            self._accessories_state.accessories, config_num
-        )
+        self._accessories_state = AccessoriesState(self._accessories_state.accessories, config_num)
         self._callback_and_save_config_changed(self.config_num)
 
-    async def _process_disconnected_events(self):
+    def _process_disconnected_events(self):
         """Process any events that happened while we were disconnected.
 
         We don't disconnect in IP so there is no need to do anything here.
@@ -456,9 +490,7 @@ class IpPairing(ZeroconfPairing):
                             return True
         return False
 
-    async def add_pairing(
-        self, additional_controller_pairing_identifier, ios_device_ltpk, permissions
-    ):
+    async def add_pairing(self, additional_controller_pairing_identifier, ios_device_ltpk, permissions):
         await self._ensure_connected()
 
         if permissions == "User":
@@ -557,4 +589,5 @@ class IpPairing(ZeroconfPairing):
         super()._async_description_update(description)
 
         # If we are not connected, or are in the process of reconnecting, hasten the process
-        self.connection.reconnect_soon()
+        if not self._shutdown:
+            self.connection.reconnect_soon()
