@@ -15,13 +15,22 @@
 #
 
 __all__ = [
-    "ChaCha20Poly1305Encryptor",
+    "NONCE_PADDING",
+    "PACK_NONCE",
     "ChaCha20Poly1305Decryptor",
-    "hkdf_derive",
+    "ChaCha20Poly1305Encryptor",
+    "DecryptionError",
     "SrpClient",
     "SrpServer",
+    "hkdf_derive",
 ]
 
-from .chacha20poly1305 import ChaCha20Poly1305Decryptor, ChaCha20Poly1305Encryptor
+from .chacha20poly1305 import (
+    NONCE_PADDING,
+    PACK_NONCE,
+    ChaCha20Poly1305Decryptor,
+    ChaCha20Poly1305Encryptor,
+    DecryptionError,
+)
 from .hkdf import hkdf_derive
 from .srp import SrpClient, SrpServer
