@@ -15,33 +15,33 @@
 #
 from __future__ import annotations
 
-from collections.abc import Iterable, Iterator
+from collections.abc import Iterator
 from dataclasses import dataclass
 from enum import Enum
 from typing import Any
 
-import aiohomekit.hkjson as hkjson
+from aiohomekit import hkjson
 from aiohomekit.protocol.statuscodes import to_status_code
 from aiohomekit.uuid import normalize_uuid
 
 from . import entity_map
 from .categories import Categories
 from .characteristics import (
+    EVENT_CHARACTERISTICS,
     Characteristic,
     CharacteristicFormats,
     CharacteristicPermissions,
     CharacteristicsTypes,
 )
 from .feature_flags import FeatureFlags
-from .mixin import get_id
 from .services import Service, ServicesTypes
 
 __all__ = [
-    "Categories",
-    "CharacteristicPermissions",
-    "CharacteristicFormats",
-    "FeatureFlags",
     "Accessory",
+    "Categories",
+    "CharacteristicFormats",
+    "CharacteristicPermissions",
+    "FeatureFlags",
     "Service",
     "ServiceTypes",
     "Transport",
@@ -60,17 +60,26 @@ class Transport(Enum):
 
 
 class Services:
+    """Represents a list of HomeKit services."""
+
     def __init__(self):
+        """Initialize a new list of services."""
         self._services: list[Service] = []
         self._iid_to_service: dict[int, Service] = {}
         self._type_to_service: dict[str, Service] = {}
 
     def __iter__(self) -> Iterator[Service]:
+        """Iterate over all services."""
         return iter(self._services)
 
     def iid(self, iid: int) -> Service:
+        """Return the service with the given iid, raising KeyError if it does not exist."""
         return self._iid_to_service[iid]
 
+    def iid_or_none(self, iid: int) -> Service | None:
+        """Return the service with the given iid or None if it does not exist."""
+        return self._iid_to_service.get(iid)
+
     def get_char_by_iid(self, iid: int) -> Characteristic | None:
         """Get a characteristic by iid."""
         for service in self._services:
@@ -86,7 +95,8 @@ class Services:
         parent_service: Service = None,
         child_service: Service = None,
         order_by: list[str] | None = None,
-    ) -> Iterable[Service]:
+    ) -> Iterator[Service]:
+        """Filter services by type and characteristics."""
         matches = iter(self._services)
 
         if service_type:
@@ -95,9 +105,7 @@ class Services:
 
         if characteristics:
             for characteristic, value in characteristics.items():
-                matches = filter(
-                    lambda service: service.value(characteristic) == value, matches
-                )
+                matches = filter(lambda service: service.value(characteristic) == value, matches)
 
         if parent_service:
             matches = filter(lambda service: service in parent_service.linked, matches)
@@ -108,9 +116,7 @@ class Services:
         if order_by:
             matches = sorted(
                 matches,
-                key=lambda service: tuple(
-                    service.value(char_type) for char_type in order_by
-                ),
+                key=lambda service: tuple(service.value(char_type) for char_type in order_by),
             )
 
         return matches
@@ -122,7 +128,8 @@ class Services:
         characteristics: dict[str, str] = None,
         parent_service: Service = None,
         child_service: Service = None,
-    ) -> Service:
+    ) -> Service | None:
+        """Get the first service."""
         if (
             service_type is not None
             and characteristics is None
@@ -143,7 +150,8 @@ class Services:
         except StopIteration:
             return None
 
-    def append(self, service: Service):
+    def append(self, service: Service) -> None:
+        """Add a service to the list of services."""
         self._services.append(service)
         self._iid_to_service[service.iid] = service
         if service.type not in self._type_to_service:
@@ -151,17 +159,23 @@ class Services:
 
 
 class Characteristics:
+    """Represents a list of HomeKit characteristics."""
+
     def __init__(self, services: Services) -> None:
+        """Initialize a new list of characteristics."""
         self._services = services
 
     def iid(self, iid: int) -> Characteristic | None:
+        """Return the characteristic with the given iid, return None if it does not exist."""
         return self._services.get_char_by_iid(iid)
 
 
 class Accessory:
-    def __init__(self) -> None:
+    """Represents a HomeKit accessory."""
+
+    def __init__(self, aid: int) -> None:
         """Initialize a new accessory."""
-        self.aid = get_id()
+        self.aid = aid
         self._next_id = 0
         self.services = Services()
         self.characteristics = Characteristics(self.services)
@@ -170,6 +184,7 @@ class Accessory:
     @classmethod
     def create_with_info(
         cls,
+        aid: int,
         name: str,
         manufacturer: str,
         model: str,
@@ -181,7 +196,7 @@ class Accessory:
         This method should only be used for testing purposes as it assigns
         the next available ids to the accessory and services.
         """
-        self = cls()
+        self = cls(aid)
 
         accessory_info = self.add_service(ServicesTypes.ACCESSORY_INFORMATION)
         accessory_info.add_char(CharacteristicsTypes.IDENTIFY, description="Identify")
@@ -189,9 +204,7 @@ class Accessory:
         accessory_info.add_char(CharacteristicsTypes.MANUFACTURER, value=manufacturer)
         accessory_info.add_char(CharacteristicsTypes.MODEL, value=model)
         accessory_info.add_char(CharacteristicsTypes.SERIAL_NUMBER, value=serial_number)
-        accessory_info.add_char(
-            CharacteristicsTypes.FIRMWARE_REVISION, value=firmware_revision
-        )
+        accessory_info.add_char(CharacteristicsTypes.FIRMWARE_REVISION, value=firmware_revision)
 
         return self
 
@@ -206,34 +219,37 @@ class Accessory:
 
     @property
     def name(self) -> str:
+        """Return the name of the accessory."""
         return self.accessory_information.value(CharacteristicsTypes.NAME, "")
 
     @property
     def manufacturer(self) -> str:
+        """Return the manufacturer of the accessory."""
         return self.accessory_information.value(CharacteristicsTypes.MANUFACTURER, "")
 
     @property
     def model(self) -> str | None:
+        """Return the model of the accessory."""
         return self.accessory_information.value(CharacteristicsTypes.MODEL, "")
 
     @property
     def serial_number(self) -> str:
+        """Return the serial number of the accessory."""
         return self.accessory_information.value(CharacteristicsTypes.SERIAL_NUMBER, "")
 
     @property
     def firmware_revision(self) -> str:
-        return self.accessory_information.value(
-            CharacteristicsTypes.FIRMWARE_REVISION, ""
-        )
+        """Return the firmware revision of the accessory."""
+        return self.accessory_information.value(CharacteristicsTypes.FIRMWARE_REVISION, "")
 
     @property
     def hardware_revision(self) -> str:
-        return self.accessory_information.value(
-            CharacteristicsTypes.HARDWARE_REVISION, ""
-        )
+        """Return the hardware revision of the accessory."""
+        return self.accessory_information.value(CharacteristicsTypes.HARDWARE_REVISION, "")
 
     @property
     def available(self) -> bool:
+        """Return True if the accessory is available."""
         return all(s.available for s in self.services)
 
     @property
@@ -251,13 +267,11 @@ class Accessory:
 
     @classmethod
     def create_from_dict(cls, data: dict[str, Any]) -> Accessory:
-        accessory = cls()
-        accessory.aid = data["aid"]
+        """Create an accessory from a dict."""
+        accessory = cls(data["aid"])
 
         for service_data in data["services"]:
-            service = accessory.add_service(
-                service_data["type"], iid=service_data["iid"], add_required=False
-            )
+            service = accessory.add_service(service_data["type"], iid=service_data["iid"], add_required=False)
             for char_data in service_data["characteristics"]:
                 kwargs = {
                     "perms": char_data["perms"],
@@ -285,21 +299,23 @@ class Accessory:
                 if "disconnected_events" in char_data:
                     kwargs["disconnected_events"] = char_data["disconnected_events"]
 
-                char = service.add_char(
-                    char_data["type"], iid=char_data["iid"], **kwargs
-                )
+                char = service.add_char(char_data["type"], iid=char_data["iid"], **kwargs)
                 if char_data.get("value") is not None:
                     char.set_value(char_data["value"])
 
         for service_data in data["services"]:
             for linked_service in service_data.get("linked", []):
-                accessory.services.iid(service_data["iid"]).add_linked_service(
-                    accessory.services.iid(linked_service)
-                )
+                # https://github.com/home-assistant/core/issues/100160
+                # The Schlage Encode Plus can contain a 0 in this list which we have to ignore
+                if linked_service:
+                    accessory.services.iid(service_data["iid"]).add_linked_service(
+                        accessory.services.iid(linked_service)
+                    )
 
         return accessory
 
     def get_next_id(self) -> int:
+        """Return the next available id for a service."""
         self._next_id += 1
         return self._next_id
 
@@ -310,31 +326,33 @@ class Accessory:
         add_required: bool = False,
         iid: int | None = None,
     ) -> Service:
-        service = Service(
-            self, service_type, name=name, add_required=add_required, iid=iid
-        )
+        """Add a service to the accessory."""
+        service = Service(self, service_type, name=name, add_required=add_required, iid=iid)
         self.services.append(service)
         return service
 
-    def to_accessory_and_service_list(self):
-        services_list = []
-        for s in self.services:
-            services_list.append(s.to_accessory_and_service_list())
-        d = {"aid": self.aid, "services": services_list}
-        return d
+    def to_accessory_and_service_list(self) -> dict[str, Any]:
+        """Serialize the accessory to a dict."""
+        return {
+            "aid": self.aid,
+            "services": [s.to_accessory_and_service_list() for s in self.services],
+        }
 
 
 class Accessories:
+    """Represents a list of HomeKit accessories."""
+
     accessories: list[Accessory]
 
     def __init__(self) -> None:
+        """Initialize a new list of accessories."""
         self.accessories = []
         self._aid_to_accessory: dict[int, Accessory] = {}
 
     def __iter__(self) -> Iterator[Accessory]:
         return iter(self.accessories)
 
-    def __getitem__(self, idx) -> Accessory:
+    def __getitem__(self, idx: int) -> Accessory:
         return self.accessories[idx]
 
     @classmethod
@@ -350,39 +368,50 @@ class Accessories:
         return self
 
     def add_accessory(self, accessory: Accessory) -> None:
+        """Add an accessory to the list of accessories."""
         self.accessories.append(accessory)
         self._aid_to_accessory[accessory.aid] = accessory
 
     def serialize(self) -> entity_map.Accesories:
-        accessories_list = []
-        for a in self.accessories:
-            accessories_list.append(a.to_accessory_and_service_list())
-        return accessories_list
+        """Serialize the accessories to a list of dicts."""
+        return [a.to_accessory_and_service_list() for a in self.accessories]
 
     def to_accessory_and_service_list(self) -> dict[str, entity_map.Accesories]:
-        d = {"accessories": self.serialize()}
-        return hkjson.dumps(d)
+        return hkjson.dumps({"accessories": self.serialize()})
 
     def aid(self, aid: int) -> Accessory:
+        """Return the accessory with the given aid, raising KeyError if it does not exist."""
         return self._aid_to_accessory[aid]
 
+    def aid_or_none(self, aid: int) -> Accessory | None:
+        """Return the accessory with the given aid or None if it does not exist."""
+        return self._aid_to_accessory.get(aid)
+
     def has_aid(self, aid: int) -> bool:
+        """Return True if the given aid exists."""
         return aid in self._aid_to_accessory
 
-    def process_changes(self, changes: dict[tuple[int, int], Any]) -> None:
-        for (aid, iid), value in changes.items():
-            accessory = self.aid(aid)
-            if not accessory:
-                continue
+    def process_changes(self, changes: dict[tuple[int, int], dict[str, Any]]) -> set[tuple[int, int]]:
+        """Process changes from a HomeKit controller.
 
-            char = accessory.characteristics.iid(iid)
-            if not char:
+        Returns a set of the changes that were applied.
+        """
+        changed: set[tuple[int, int]] = set()
+        for aid_iid, value in changes.items():
+            (aid, iid) = aid_iid
+            if not (char := self.aid(aid).characteristics.iid(iid)):
                 continue
 
             if "value" in value:
-                char.set_value(value["value"])
+                if char.set_value(value["value"]) or char.type in EVENT_CHARACTERISTICS:
+                    changed.add(aid_iid)
 
+            previous_status = char.status
             char.status = to_status_code(value.get("status", 0))
+            if previous_status != char.status:
+                changed.add(aid_iid)
+
+        return changed
 
 
 @dataclass
