@@ -19,6 +19,7 @@ from .characteristic_formats import CharacteristicFormats
 from .characteristic_types import CharacteristicsTypes
 from .const import (
     ActivationStateValues,
+    CurrentFanStateValues,
     CurrentHeaterCoolerStateValues,
     CurrentMediaStateValues,
     HeatingCoolingCurrentValues,
@@ -29,6 +30,7 @@ from .const import (
     ProgramModeValues,
     RemoteKeyValues,
     SwingModeValues,
+    TargetFanStateValues,
     TargetHeaterCoolerStateValues,
     TargetMediaStateValues,
     ValveTypeValues,
@@ -37,26 +39,38 @@ from .permissions import CharacteristicPermissions
 from .types import CharacteristicShortUUID, CharacteristicUUID
 from .units import CharacteristicUnits
 
+EVENT_CHARACTERISTICS = {
+    CharacteristicsTypes.INPUT_EVENT,
+    CharacteristicsTypes.BUTTON_EVENT,
+}
+# These characteristics are marked as [pr,ev] but make no sense to poll.
+#
+# Doing so can cause phantom triggers.
+
+
 __all__ = [
+    "EVENT_CHARACTERISTICS",
+    "ActivationStateValues",
     "Characteristic",
     "CharacteristicFormats",
     "CharacteristicPermissions",
-    "CharacteristicsTypes",
+    "CharacteristicShortUUID",
+    "CharacteristicUUID",
     "CharacteristicUnits",
-    "TargetMediaStateValues",
+    "CharacteristicsTypes",
+    "CurrentFanStateValues",
+    "CurrentHeaterCoolerStateValues",
     "CurrentMediaStateValues",
-    "RemoteKeyValues",
-    "InputEventValues",
     "HeatingCoolingCurrentValues",
     "HeatingCoolingTargetValues",
-    "CharacteristicUUID",
-    "CharacteristicShortUUID",
     "InUseValues",
+    "InputEventValues",
     "IsConfiguredValues",
     "ProgramModeValues",
-    "ValveTypeValues",
-    "ActivationStateValues",
+    "RemoteKeyValues",
     "SwingModeValues",
-    "CurrentHeaterCoolerStateValues",
+    "TargetFanStateValues",
     "TargetHeaterCoolerStateValues",
+    "TargetMediaStateValues",
+    "ValveTypeValues",
 ]
