@@ -71,10 +71,9 @@ def strtobool(val):
     val = val.lower()
     if val in ("y", "yes", "t", "true", "on", "1"):
         return 1
-    elif val in ("n", "no", "f", "false", "off", "0"):
+    if val in ("n", "no", "f", "false", "off", "0"):
         return 0
-    else:
-        raise ValueError(f"invalid truth value {val!r}")
+    raise ValueError(f"invalid truth value {val!r}")
 
 
 class Characteristic:
@@ -93,23 +92,15 @@ class Characteristic:
     def __init__(self, service: Service, characteristic_type: str, **kwargs) -> None:
         self.service = service
         self.type = normalize_uuid(characteristic_type)
-        self.iid = self._get_configuration(
-            kwargs, "iid", service.accessory.get_next_id()
-        )
+        self.iid = self._get_configuration(kwargs, "iid", service.accessory.get_next_id())
 
-        self.perms = self._get_configuration(
-            kwargs, "perms", [CharacteristicPermissions.paired_read]
-        )
+        self.perms = self._get_configuration(kwargs, "perms", [CharacteristicPermissions.paired_read])
         self.format = self._get_configuration(kwargs, "format", None)
 
         self.ev = None
         self.handle = self._get_configuration(kwargs, "handle", None)
-        self.broadcast_events = self._get_configuration(
-            kwargs, "broadcast_events", None
-        )
-        self.disconnected_events = self._get_configuration(
-            kwargs, "disconnected_events", None
-        )
+        self.broadcast_events = self._get_configuration(kwargs, "broadcast_events", None)
+        self.disconnected_events = self._get_configuration(kwargs, "disconnected_events", None)
         self.description = self._get_configuration(kwargs, "description", None)
         self.unit = self._get_configuration(kwargs, "unit", None)
         self.minValue = self._get_configuration(kwargs, "min_value", None)
@@ -175,20 +166,31 @@ class Characteristic:
     def set_events(self, new_val: Any) -> None:
         self.ev = new_val
 
-    def set_value(self, new_val: Any) -> None:
+    def set_value(self, new_val: Any) -> bool:
         """
         This function sets the value of this characteristic.
+
+        Returns True if the value has changed, False otherwise.
         """
 
         if self.format == CharacteristicFormats.bool:
             # Device might return 1 or 0, so lets case to True/False
             new_val = bool(new_val)
 
+        changed = self._value != new_val
         self._value = new_val
+        return changed
 
     @property
     def value(self) -> Any:
-        extra_data = characteristics.get(self.type, {})
+        """Get the value of the characteristic.
+
+        If the characteristic is a tlv8, decodes the struct
+
+        If the characteristic is an enum, returns the enum value
+        """
+        if not (extra_data := characteristics.get(self.type)):
+            return self._value
 
         if self.format == CharacteristicFormats.tlv8:
             new_val = base64.b64decode(self._value)
@@ -196,8 +198,7 @@ class Characteristic:
             if struct:
                 if extra_data.get("array"):
                     return [struct.decode(new_val) for new_val in tlv_array(new_val)]
-                else:
-                    return struct.decode(new_val)
+                return struct.decode(new_val)
 
         if enum := extra_data.get("enum"):
             return enum(self._value)
@@ -374,9 +375,7 @@ def check_convert_value(val: str, char: Characteristic) -> Any:
 
                 # We use to_integral_value() here rather than round as it respsects
                 # ctx.rounding
-                val = offset + (
-                    ((val - offset) / min_step).to_integral_value() * min_step
-                )
+                val = offset + (((val - offset) / min_step).to_integral_value() * min_step)
 
         if char.format in INTEGER_TYPES:
             val = int(val.to_integral_value())
