@@ -16,8 +16,11 @@
 
 __all__ = [
     "Service",
+    "ServiceShortUUID",
+    "ServiceUUID",
     "ServicesTypes",
 ]
 
 from .service import Service
 from .service_types import ServicesTypes
+from .types import ServiceShortUUID, ServiceUUID
