@@ -16,10 +16,10 @@
 
 from __future__ import annotations
 
-from collections.abc import Iterable
-from enum import Enum
 import logging
 import struct
+from collections.abc import Iterable
+from enum import Enum
 
 from aiohomekit.enum import EnumWithDescription
 
@@ -39,7 +39,6 @@ STRUCT_BB_UNPACK = struct.Struct("<BB").unpack
 
 
 class OpCode(Enum):
-
     CHAR_SIG_READ = 0x01
     CHAR_WRITE = 0x02
     CHAR_READ = 0x03
@@ -51,7 +50,6 @@ class OpCode(Enum):
 
 
 class PDUStatus(EnumWithDescription):
-
     SUCCESS = 0, "Success"
     UNSUPPORTED_PDU = 1, "Unsupported PDU"
     MAX_PROCEDURES = 2, "Max procedures"
@@ -109,16 +107,12 @@ def decode_pdu(expected_tid: int, data: bytes) -> tuple[PDUStatus, bool, bytes]:
     )
 
     if tid != expected_tid:
-        raise ValueError(
-            f"Expected transaction {expected_tid} but got transaction {tid}"
-        )
+        raise ValueError(f"Expected transaction {expected_tid} but got transaction {tid}")
 
     if status != PDUStatus.SUCCESS:
         # We can't currently raise here or it will break the encryption
         # stream
-        logger.warning(
-            f"Transaction {tid} failed with error {status} ({status.description})"
-        )
+        logger.warning(f"Transaction {tid} failed with error {status} ({status.description})")
 
     if len(data) < 5:
         return status, 0, b""
@@ -144,8 +138,6 @@ def decode_pdu_continuation(expected_tid, data):
         raise ValueError("Expected continuation flag but isn't set")
 
     if tid != expected_tid:
-        raise ValueError(
-            f"Expected transaction {expected_tid} but got transaction {tid}"
-        )
+        raise ValueError(f"Expected transaction {expected_tid} but got transaction {tid}")
 
     return data[2:]
