@@ -15,9 +15,9 @@
 #
 from __future__ import annotations
 
+import logging
 from binascii import hexlify
 from collections.abc import Generator
-import logging
 from typing import Any, Callable
 
 from cryptography import exceptions as cryptography_exceptions
@@ -25,8 +25,10 @@ from cryptography.hazmat.primitives import serialization
 from cryptography.hazmat.primitives.asymmetric import ed25519, x25519
 
 from aiohomekit.crypto import (
+    NONCE_PADDING,
     ChaCha20Poly1305Decryptor,
     ChaCha20Poly1305Encryptor,
+    DecryptionError,
     SrpClient,
     hkdf_derive,
 )
@@ -58,18 +60,17 @@ def error_handler(error: bytearray, stage: str):
     """
     if error == TLV.kTLVError_Unavailable:
         raise UnavailableError(stage)
-    elif error == TLV.kTLVError_Authentication:
+    if error == TLV.kTLVError_Authentication:
         raise AuthenticationError(stage)
-    elif error == TLV.kTLVError_Backoff:
+    if error == TLV.kTLVError_Backoff:
         raise BackoffError(stage)
-    elif error == TLV.kTLVError_MaxPeers:
+    if error == TLV.kTLVError_MaxPeers:
         raise MaxPeersError(stage)
-    elif error == TLV.kTLVError_MaxTries:
+    if error == TLV.kTLVError_MaxTries:
         raise MaxTriesError(stage)
-    elif error == TLV.kTLVError_Busy:
+    if error == TLV.kTLVError_Busy:
         raise BusyError(stage)
-    else:
-        raise InvalidError(stage)
+    raise InvalidError(stage)
 
 
 def handle_state_step(tlv_dict, expected_state):
@@ -90,9 +91,7 @@ def handle_state_step(tlv_dict, expected_state):
 
 def perform_pair_setup_part1(
     with_auth: bool = True,
-) -> Generator[
-    tuple[list[tuple[int, bytearray]], list[int]], None, tuple[bytearray, bytearray]
-]:
+) -> Generator[tuple[list[tuple[int, bytearray]], list[int]], None, tuple[bytearray, bytearray]]:
     """
     Performs a pair setup operation as described in chapter 4.7 page 39 ff.
 
@@ -141,17 +140,14 @@ def perform_pair_setup_part1(
 def validate_mfi(session_key, response_tlv):
     # If pairing method is PairSetupWithAuth there should be an EncryptedData TLV in M4
     # It should have a signature and a certificate from an Apple secure co-processor.
-    decrypted = ChaCha20Poly1305Decryptor(session_key).decrypt(
-        b"",
-        b"PS-Msg04",
-        bytes([0, 0, 0, 0]),
-        response_tlv[TLV.kTLVType_EncryptedData],
-    )
-
-    if not decrypted:
-        logger.debug(
-            "Device returned kTLVType_EncryptedData during M4 but could not decrypt"
+    try:
+        decrypted = ChaCha20Poly1305Decryptor(session_key).decrypt(
+            b"",
+            NONCE_PADDING + b"PS-Msg04",
+            bytes(response_tlv[TLV.kTLVType_EncryptedData]),
         )
+    except DecryptionError:
+        logger.debug("Device returned kTLVType_EncryptedData during M4 but could not decrypt")
         return
 
     sub_tlv = TLV.decode_bytes(decrypted)
@@ -163,17 +159,13 @@ def validate_mfi(session_key, response_tlv):
         return
 
     if TLV.kTLVType_Certificate not in sub_tlv:
-        logger.debug(
-            "QUIRK: M4: Device returned kTLVType_Signature but not kTLVType_Certificate"
-        )
+        logger.debug("QUIRK: M4: Device returned kTLVType_Signature but not kTLVType_Certificate")
         return
 
     # Certificate appears to be X509 in DER format but with some sort of PKCS7 pre-amble.
     # cryptography doesn't seem to support that yet.
 
-    logger.debug(
-        "Found seemingly valid MFI kTLVType_Signature; we don't validate this yet"
-    )
+    logger.debug("Found seemingly valid MFI kTLVType_Signature; we don't validate this yet")
 
 
 def perform_pair_setup_part2(
@@ -275,7 +267,7 @@ def perform_pair_setup_part2(
     # https://github.com/KhaosT/HAP-NodeJS/blob/2ea9d761d9bd7593dd1949fec621ab085af5e567/lib/HAPServer.js
     # function handlePairStepFive calling encryption.encryptAndSeal
     encrypted_data_with_auth_tag = ChaCha20Poly1305Encryptor(session_key).encrypt(
-        b"", b"PS-Msg05", bytes([0, 0, 0, 0]), sub_tlv_b
+        b"", NONCE_PADDING + b"PS-Msg05", bytes(sub_tlv_b)
     )
 
     response_tlv = [
@@ -299,16 +291,16 @@ def perform_pair_setup_part2(
     if TLV.kTLVType_EncryptedData not in response_tlv:
         raise InvalidError("M6: Encrypted data not sent be accessory")
 
-    decrypted_data = ChaCha20Poly1305Decryptor(session_key).decrypt(
-        b"",
-        b"PS-Msg06",
-        bytes([0, 0, 0, 0]),
-        response_tlv[TLV.kTLVType_EncryptedData],
-    )
-    if decrypted_data is False:
+    try:
+        decrypted_data = ChaCha20Poly1305Decryptor(session_key).decrypt(
+            b"",
+            NONCE_PADDING + b"PS-Msg06",
+            bytes(response_tlv[TLV.kTLVType_EncryptedData]),
+        )
+    except DecryptionError:
         raise IllegalData("step 7")
 
-    response_tlv = TLV.decode_bytearray(decrypted_data)
+    response_tlv = TLV.decode_bytearray(bytearray(decrypted_data))
     response_tlv = dict(response_tlv)
 
     if TLV.kTLVType_Signature not in response_tlv:
@@ -368,8 +360,7 @@ def resume_m1(
 
     auth_tag = key.encrypt(
         b"",
-        b"PR-Msg01",
-        bytes([0, 0, 0, 0]),
+        NONCE_PADDING + b"PR-Msg01",
         b"",
     )
 
@@ -409,17 +400,18 @@ def resume_m3(
     )
 
     key = ChaCha20Poly1305Decryptor(response_key)
-    plaintext = key.decrypt(
-        b"",
-        b"PR-Msg02",
-        bytes([0, 0, 0, 0]),
-        auth_tag,
-    )
+    try:
+        plaintext = key.decrypt(
+            b"",
+            NONCE_PADDING + b"PR-Msg02",
+            bytes(auth_tag),
+        )
+    except DecryptionError:
+        logger.debug("M3: Failure to resume existing session: Could not decrypt kTLVType_EncryptedData")
+        return None
 
     if plaintext != b"":
-        logger.debug(
-            "M3: Failure to resume existing session: Could not decrypt kTLVType_EncryptedData"
-        )
+        logger.debug("M3: Failure to resume existing session: Could not decrypt kTLVType_EncryptedData")
         return None
 
     shared_secret = derive(
@@ -496,22 +488,19 @@ def get_session_keys(
 
     # 1) generate shared secret
     accessorys_session_pub_key_bytes = bytes(response_tlv[TLV.kTLVType_PublicKey])
-    accessorys_session_pub_key = x25519.X25519PublicKey.from_public_bytes(
-        accessorys_session_pub_key_bytes
-    )
+    accessorys_session_pub_key = x25519.X25519PublicKey.from_public_bytes(accessorys_session_pub_key_bytes)
     shared_secret = ios_key.exchange(accessorys_session_pub_key)
 
     # 2) derive session key
-    session_key = hkdf_derive(
-        shared_secret, b"Pair-Verify-Encrypt-Salt", b"Pair-Verify-Encrypt-Info"
-    )
+    session_key = hkdf_derive(shared_secret, b"Pair-Verify-Encrypt-Salt", b"Pair-Verify-Encrypt-Info")
 
     # 3) verify auth tag on encrypted data and 4) decrypt
     encrypted = response_tlv[TLV.kTLVType_EncryptedData]
-    decrypted = ChaCha20Poly1305Decryptor(session_key).decrypt(
-        b"", b"PV-Msg02", bytes([0, 0, 0, 0]), encrypted
-    )
-    if type(decrypted) == bool and not decrypted:
+    try:
+        decrypted = ChaCha20Poly1305Decryptor(session_key).decrypt(
+            b"", NONCE_PADDING + b"PV-Msg02", bytes(encrypted)
+        )
+    except DecryptionError:
         raise InvalidAuthTagError("step 3")
     d1 = dict(TLV.decode_bytes(decrypted))
 
@@ -527,35 +516,25 @@ def get_session_keys(
     if pairing_data["AccessoryPairingID"] != accessory_name:
         raise IncorrectPairingIdError("step 3")
 
-    accessory_ltpk = ed25519.Ed25519PublicKey.from_public_bytes(
-        bytes.fromhex(pairing_data["AccessoryLTPK"])
-    )
+    accessory_ltpk = ed25519.Ed25519PublicKey.from_public_bytes(bytes.fromhex(pairing_data["AccessoryLTPK"]))
 
     # 6) verify accessory's signature
     accessory_sig = d1[TLV.kTLVType_Signature]
     accessory_session_pub_key_bytes = response_tlv[TLV.kTLVType_PublicKey]
-    accessory_info = (
-        accessory_session_pub_key_bytes + accessory_name.encode() + ios_key_pub
-    )
+    accessory_info = accessory_session_pub_key_bytes + accessory_name.encode() + ios_key_pub
     try:
         accessory_ltpk.verify(bytes(accessory_sig), bytes(accessory_info))
     except cryptography_exceptions.InvalidSignature:
         raise InvalidSignatureError("step 3")
 
     # 7) create iOSDeviceInfo
-    ios_device_info = (
-        ios_key_pub
-        + pairing_data["iOSPairingId"].encode()
-        + accessorys_session_pub_key_bytes
-    )
+    ios_device_info = ios_key_pub + pairing_data["iOSPairingId"].encode() + accessorys_session_pub_key_bytes
 
     # 8) sign iOSDeviceInfo with long term secret key
     ios_device_ltsk_h = pairing_data["iOSDeviceLTSK"]
     # ios_device_ltpk_h = pairing_data["iOSDeviceLTPK"]
 
-    ios_device_ltsk = ed25519.Ed25519PrivateKey.from_private_bytes(
-        bytes.fromhex(ios_device_ltsk_h)
-    )
+    ios_device_ltsk = ed25519.Ed25519PrivateKey.from_private_bytes(bytes.fromhex(ios_device_ltsk_h))
     # ios_device_ltsk = ed25519.SigningKey(
     #    bytes.fromhex(ios_device_ltsk_h) + bytes.fromhex(ios_device_ltpk_h)
     # )
@@ -571,7 +550,7 @@ def get_session_keys(
 
     # 10) encrypt and sign
     encrypted_data_with_auth_tag = ChaCha20Poly1305Encryptor(session_key).encrypt(
-        b"", b"PV-Msg03", bytes([0, 0, 0, 0]), sub_tlv
+        b"", NONCE_PADDING + b"PV-Msg03", bytes(sub_tlv)
     )
 
     # 11) create tlv
