@@ -14,8 +14,12 @@
 # limitations under the License.
 #
 
+import logging
+
 from aiohomekit.enum import EnumWithDescription
 
+logger = logging.getLogger(__name__)
+
 
 class HapStatusCode(EnumWithDescription):
     SUCCESS = 0, "This specifies a success for the request."
@@ -37,12 +41,19 @@ class HapStatusCode(EnumWithDescription):
     INVALID_VALUE = -70410, "Accessory received an invalid value in a write request."
     INSUFFICIENT_AUTH = -70411, "Insufficient Authorization."
     NOT_ALLOWED_IN_CURRENT_STATE = -70412, "Not allowed in current state"
+    # Special status for unknown/undefined error codes
+    UNKNOWN = -1, "Unknown or undefined status code"
 
 
 def to_status_code(status_code: int) -> HapStatusCode:
     # Some HAP implementations return positive values for error code (myq)
-    status_code = abs(status_code) * -1
-    return HapStatusCode(status_code)
+    normalized = abs(status_code) * -1
+    try:
+        return HapStatusCode(normalized)
+    except ValueError:
+        # Return UNKNOWN for undefined status codes
+        logger.debug("Unknown HAP status code: %s (normalized: %s)", status_code, normalized)
+        return HapStatusCode.UNKNOWN
 
 
 class _HapBleStatusCodes:
