@@ -16,10 +16,10 @@
 from __future__ import annotations
 
 import base64
+import logging
 from collections.abc import AsyncIterable
 from dataclasses import dataclass
 from datetime import timedelta
-import logging
 
 from aiohomekit import exceptions
 from aiohomekit.characteristic_cache import CharacteristicCacheMemory
@@ -34,7 +34,10 @@ from aiohomekit.controller.abstract import (
 from aiohomekit.exceptions import AccessoryNotFoundError
 from aiohomekit.model import Accessories, AccessoriesState, Transport
 from aiohomekit.model.categories import Categories
-from aiohomekit.model.characteristics import CharacteristicsTypes
+from aiohomekit.model.characteristics import Characteristic, CharacteristicsTypes
+from aiohomekit.model.characteristics.characteristic_formats import (
+    CharacteristicFormats,
+)
 from aiohomekit.model.status_flags import StatusFlags
 from aiohomekit.protocol.statuscodes import HapStatusCode
 from aiohomekit.uuid import normalize_uuid
@@ -61,9 +64,7 @@ class FakeDescription:
 class FakeDiscovery(AbstractDiscovery):
     description = FakeDescription()
 
-    def __init__(
-        self, controller: FakeController, device_id: str, accessories: Accessories
-    ):
+    def __init__(self, controller: FakeController, device_id: str, accessories: Accessories):
         self.controller = controller
         self.accessories = accessories
 
@@ -150,33 +151,39 @@ class PairingTester:
             uuid = normalize_uuid(uuid)
 
             if uuid not in service:
-                raise RuntimeError(
-                    f"Unexpected characteristic {uuid!r} applied to service {name!r}"
-                )
+                raise RuntimeError(f"Unexpected characteristic {uuid!r} applied to service {name!r}")
 
-            char = service[uuid]
-            char.set_value(value)
-            changed.append((char.service.accessory.aid, char.iid))
+            char: Characteristic = service[uuid]
+            changed.append((char.service.accessory.aid, char.iid, value))
 
         self._send_events(changed)
 
     def update_aid_iid(self, characteristics):
-        changed = []
-        for aid, iid, value in characteristics:
-            self.characteristics[(aid, iid)].set_value(value)
-            changed.append((aid, iid))
+        self._send_events(characteristics)
 
-        self._send_events(changed)
+    def set_aid_iid_status(self, aid_iid_statuses: list[tuple[int, int, int]]):
+        """Set status for an aid iid pair."""
+        event = {(aid, iid): {"status": status} for aid, iid, status in aid_iid_statuses}
 
-    def _send_events(self, changed):
+        if not event:
+            return
+
+        for listener in self.pairing.listeners:
+            try:
+                listener(event)
+            except Exception:
+                _LOGGER.exception("Unhandled error when processing event")
+
+    def _send_events(self, characteristics):
         if not self.events_enabled:
             return
 
         event = {}
-        for aid, iid in changed:
-            if (aid, iid) not in self.pairing.subscriptions:
-                continue
-            event[(aid, iid)] = {"value": self.characteristics[(aid, iid)].get_value()}
+        for aid, iid, value in characteristics:
+            char: Characteristic = self.characteristics[(aid, iid)]
+            if char.format == CharacteristicFormats.bool:
+                value = bool(value)
+            event[(aid, iid)] = {"value": value}
 
         if not event:
             return
@@ -270,9 +277,7 @@ class FakePairing(AbstractPairing):
 
     async def _process_config_changed(self, config_num: int) -> None:
         await self.list_accessories_and_characteristics()
-        self._accessories_state = AccessoriesState(
-            self._accessories_state.accessories, config_num
-        )
+        self._accessories_state = AccessoriesState(self._accessories_state.accessories, config_num)
         self._callback_and_save_config_changed(config_num)
 
     async def _process_disconnected_events(self):
@@ -318,7 +323,10 @@ class FakePairing(AbstractPairing):
         self,
         dataset: str,
     ) -> None:
-        pass
+        # This ultimately needs refactoring so that we can have multiple test transports loaded
+        # rather than patching this one to be COAP.
+        self.pairing_data["Connection"] = "CoAP"
+        self.controller.transport_type = TransportType.COAP
 
     async def image(self, accessory: int, width: int, height: int) -> bytes:
         self._ensure_connected()
@@ -341,9 +349,7 @@ class FakeController(AbstractController):
 
     transport_type = TransportType.IP
 
-    def __init__(
-        self, async_zeroconf_instance=None, char_cache=None, bleak_scanner_instance=None
-    ):
+    def __init__(self, async_zeroconf_instance=None, char_cache=None, bleak_scanner_instance=None):
         super().__init__(char_cache=char_cache or CharacteristicCacheMemory())
 
     def add_device(self, accessories):
@@ -357,9 +363,7 @@ class FakeController(AbstractController):
 
     async def add_paired_device(self, accessories: Accessories, alias: str = None):
         discovery = self.add_device(accessories)
-        finish_pairing = await discovery.async_start_pairing(
-            alias or discovery.description.id
-        )
+        finish_pairing = await discovery.async_start_pairing(alias or discovery.description.id)
         return await finish_pairing(discovery.pairing_code)
 
     async def async_start(self) -> None:
@@ -368,9 +372,7 @@ class FakeController(AbstractController):
     async def async_stop(self) -> None:
         self.started = False
 
-    async def async_discover(
-        self, max_seconds: int = 10
-    ) -> AsyncIterable[AbstractDiscovery]:
+    async def async_discover(self, max_seconds: int = 10) -> AsyncIterable[AbstractDiscovery]:
         for discovery in self.discoveries.values():
             yield discovery
 
@@ -380,6 +382,9 @@ class FakeController(AbstractController):
         except KeyError:
             raise AccessoryNotFoundError(device_id)
 
+    async def async_reachable(self, device_id: str, timeout=10) -> bool:
+        return True
+
     async def remove_pairing(self, alias: str) -> None:
         pairing = self.aliases[alias]
         del self.pairings[self.aliases[alias].id]
