@@ -1,11 +1,11 @@
 from __future__ import annotations
 
+import enum
+import struct
 from collections import abc
 from collections.abc import Iterable, Sequence
 from dataclasses import field, fields
-import enum
-from functools import lru_cache
-import struct
+from functools import cache, lru_cache
 from typing import Any, Callable, ClassVar, Generic, TypeVar, _GenericAlias
 
 SerializerCallback = Callable[[type, Any], bytes]
@@ -54,14 +54,10 @@ def get_origin(tp):
 class TlvParseException(Exception):
     """Raised upon parse error with some TLV"""
 
-    pass
-
 
 class TlvSerializeException(Exception):
     """Raised upon parse error with some TLV"""
 
-    pass
-
 
 def tlv_iterator(encoded_struct: bytes) -> Iterable:
     offset = 0
@@ -74,6 +70,12 @@ def tlv_iterator(encoded_struct: bytes) -> Iterable:
         # Iterate until the type changes
         while length == 255:
             peek_offset = offset + 2 + length
+            if peek_offset >= len(encoded_struct):
+                # https://github.com/home-assistant/core/issues/100160
+                # Schlage Encode Plus seems to incorrectly serialize "linked services"
+                # Recover as gracefully as possible
+                # If we break here, we'll have all the value that we received, but length might be wrong.
+                break
             if encoded_struct[peek_offset] != type:
                 break
             offset = peek_offset
@@ -274,12 +276,10 @@ class TLVStruct:
         return bytes(result)
 
     @classmethod
-    @lru_cache(maxsize=None)
+    @cache
     def _tlv_types(cls: T) -> dict:
         """Return the TLV types for this class."""
-        return {
-            field.metadata["tlv_type"]: field for field in fields(cls) if field.init
-        }
+        return {field.metadata["tlv_type"]: field for field in fields(cls) if field.init}
 
     @classmethod
     def decode(cls: T, encoded_struct: bytes) -> T:
